$title  Recalibration Routine for GTAP versions 3 and 4

*       Scale transportation demand to equal supply:

yt0("prod") = sum((i,r), vst(i,r));
yt0("demd") = sum((i,s,r),vtwr(i,s,r));
vtwr(i,s,r) = vtwr(i,s,r) * yt0("prod") / yt0("demd");

*       Recalculate import demand and export supply:

vxm(i,r) = sum(s, vxmd(i,r,s)) + vst(i,r);
vim(i,r) = sum(s, (vxmd(i,s,r) * (1+tx(i,s,r)) + vtwr(i,s,r) ) * (1+tm(i,s,r)) );

vdm(i,r) = (sum(j, vafm(j,i,r)*(1+ti(j,i,r))) + sum(f, vfm(f,i,r)*(1+tf(f,i,r))))/(1-ty(i,r)) - vxm(i,r);

vdfm(i,r) = max(0, vdm(i,r) - vdgm(i,r) - vdpm(i,r) - vdm(i,r)$cgd(i));
vifm(i,r) = max(0, vim(i,r) - vipm(i,r) - vigm(i,r));

vdfm(i,r)$(not sum(j, vafm(i,j,r))) = 0;
vifm(i,r)$(not sum(j, vafm(i,j,r))) = 0;

vifm(i,r)$(not vim(i,r)) = 0;
vigm(i,r)$(not vim(i,r)) = 0;
vipm(i,r)$(not vim(i,r)) = 0;

*       If we have no demand for an imported good, then create one so
*       that the calibration always remains feasible:

vipm(i,r)$((not (vipm(i,r)+vigm(i,r)+vifm(i,r))) and vim(i,r)) = vim(i,r);

parameter       d0, m0, invest;

m0("c",i,r) = vipm(i,r);
d0("c",i,r) = vdpm(i,r);

m0("g",i,r) = vigm(i,r);
d0("g",i,r) = vdgm(i,r);

m0("i",i,r) = vifm(i,r);
d0("i",i,r) = vdfm(i,r);

invest(r) = sum(cgd, vdm(cgd,r));

parameter dommktchk;
dommktchk(i,r) =  vdm(i,r) - sum(d, d0(d,i,r)) - invest(r)$cgd(i);
display dommktchk;

parameter fmscale;
fmscale(i,r)$(vdfm(i,r)+vifm(i,r)) = sum(j, vafm(i,j,r)) / (vdfm(i,r)+vifm(i,r)) - 1;
display fmscale;

vdfm(i,r) = vdfm(i,r) * (1 + fmscale(i,r));
vifm(i,r) = vifm(i,r) * (1 + fmscale(i,r));


m0("c",i,r) = vipm(i,r);
d0("c",i,r) = vdpm(i,r);

m0("g",i,r) = vigm(i,r);
d0("g",i,r) = vdgm(i,r);

m0("i",i,r) = vifm(i,r);
d0("i",i,r) = vdfm(i,r);

dommktchk(i,r) =  vdm(i,r) - sum(d, d0(d,i,r)) - invest(r)$cgd(i);
display dommktchk;

*       ----------------------------------------

*       Read the model used for regional balancing:

$include regbal.gms

option solprint=off,limrow=0,limcol=0;

parameter marketchk(i,r,*,*)    Market clearance condition;
option marketchk:3:2:2;

marketchk(i,r,"vdfm","before") = vdfm(i,r);
marketchk(i,r,"vifm","before") = vifm(i,r);
marketchk(i,r,"vafm","before") = sum(j, vafm(i,j,r));
marketchk(i,r,"total","before") = round(vdfm(i,r) + vifm(i,r) - sum(j, vafm(i,j,r)), 6);

loop(r,

	display vfm;

*       Identify current region:

        rb(r) = yes;

*       Set default bounds:

        vgmz.lo(i)   = 0;       vgmz.up(i)   = inf;
        vpmz.lo(i)   = 0;       vpmz.up(i)   = inf;
        vdmz.lo(i) = 0;         vdmz.up(i) = inf;
        vafmz.lo(j,i) = 0;      vafmz.up(j,i) = inf;
        vfmz.lo(f,i) = 0;       vfmz.up(f,i) = inf;
        m0z.lo(d,i) = 0;        m0z.up(d,i) = inf;
        d0z.lo(d,i) = 0;        d0z.up(d,i) = inf;
        investz.lo = 0;         investz.up = inf;

*       Initial level values:

        vgmz.l(i) = vgm(i,r);
        vpmz.l(i) = vpm(i,r);
        vdmz.l(i) = vdm(i,r);
        vafmz.l(j,i) = vafm(j,i,r);
        vfmz.l(f,i) = vfm(f,i,r);
        m0z.l(d,i) = m0(d,i,r);
        d0z.l(d,i) = d0(d,i,r);
        investz.l = invest(r);
        
*       Impose the same sparsity pattern as is
*       present in the original dataset:

        vgmz.fx(i)$(not vgm(i,r)) = 0;
        vpmz.fx(i)$(not vpm(i,r)) = 0;
        vafmz.fx(j,i)$(not vafm(j,i,r)) = 0;
        vfmz.fx( f,i)$(not vfm( f,i,r)) = 0;
        vdmz.fx(i)$(not vdm(i,r)) = 0;
        m0z.fx(d,i)$(not m0(d,i,r)) = 0;
        d0z.fx(d,i)$(not d0(d,i,r)) = 0;

*       Initialize bounds on dual variables:

        pzprofit.lo(i,rb) = -inf;       pzprofit.up(i,rb) = inf;
        pzdommkt.lo(i,rb) = -inf;       pzdommkt.up(i,rb) = inf;
        pzimport.lo(i,rb) = -inf;       pzimport.up(i,rb) = inf;
        pzinterm.lo(i,rb) = -inf;       pzinterm.up(i,rb) = inf;
        pzconsum.lo(i,rb) = -inf;       pzconsum.up(i,rb) = inf;
        pzgovern.lo(i,rb) = -inf;       pzgovern.up(i,rb) = inf;

*       Initialize levels on dual variables (what values?):

        pzprofit.l(i,rb) = 0;
        pzdommkt.l(i,rb) = 0;
        pzimport.l(i,rb) = 0;
        pzinterm.l(i,rb) = 0;
        pzconsum.l(i,rb) = 0;
        pzgovern.l(i,rb) = 0;

*       Fix duals for vacuous equations -- this drops these

        pzprofit.fx(i,rb)$(not vdm(i,r)+vxm(i,r)) = 0;
        pzdommkt.fx(i,rb)$(not vdm(i,r)) = 0;
        pzimport.fx(i,rb)$(not vim(i,r)) = 0;
        pzinterm.fx(i,rb)$(not sum(j, vafm(i,j,r))) = 0;
        pzconsum.fx(i,rb)$(not vpm(i,r)) = 0;
        pzgovern.fx(i,rb)$(not vgm(i,r)) = 0;

        solve regbalmcp using mcp;

*       Retrieve the solution:

        vgm(i,r)    = vgmz.l(i);
        vpm(i,r)    = vpmz.l(i);
        vdm(i,r)    = vdmz.l(i);
        vafm(j,i,r) = vafmz.l(j,i);
        vfm(f,i,r)  = vfmz.l(f,i);

        m0(d,i,r) = m0z.l(d,i);
        d0(d,i,r) = d0z.l(d,i);

        vipm(i,r) = m0("c",i,r);
        vigm(i,r) = m0("g",i,r);
        vdpm(i,r) = d0("c",i,r);
        vdgm(i,r) = d0("g",i,r);

        vifm(i,r) = m0("i",i,r);
        vdfm(i,r) = d0("i",i,r);

        marketchk(i,r,"vdfm","after") = vdfm(i,r);
        marketchk(i,r,"vifm","after") = vifm(i,r);
        marketchk(i,r,"vafm","after") = sum(j, vafm(i,j,r));
        marketchk(i,r,"total","after") = round(vdfm(i,r) + vifm(i,r) - sum(j, vafm(i,j,r)), 6);

        rb(r) = no;

);

display vfm;

*       Display a report of the revisions:

display marketchk;

vafm(j,i,r) = max(0, vafm(j,i,r));
vfm(f,i,r)  = max(0, vfm(f,i,r));
vxmd(i,r,s) = max(0, vxmd(i,r,s));
vtwr(i,r,s) = max(0, vtwr(i,r,s));
vst(i,r)    = max(0, vst(i,r));
vdgm(i,r)   = max(0, vdgm(i,r));
vigm(i,r)   = max(0, vigm(i,r));
vdpm(i,r)   = max(0, vdpm(i,r));
vipm(i,r)   = max(0, vipm(i,r));

