$title  Impose Gerard's Energy Data onto the GTAP Database

*	Define whether to solve calibration problems as NLP or MCP
*	with the following statement:

scalar	mcpformat /1/;

parameter	
	A1	"Energy value shares (%)--(original GTAP)",
	A2	"CO2 inventories (IEA)--mton -- GTAP with aggregate oil, coal, gas use.",
	A3	"Sectoral CO2 intensities --kg per $output -- (original GTAP with IEA total supply)",
	B1	"Coal statistics (original IEA) -- EJ",
	B2	"Crude Oil statistics (original IEA) -- EJ",
	B3	"Refined Oil statistics (original IEA) -- EJ",
	B4	"Natural gas statistics (original IEA) -- EJ",
	B5	"Electricity statistics (original IEA) -- TKWT",
	C1	"CO2 inventories (original IEA)--mton",
	C2	"Sectoral CO2 intensities (original IEA-GTAP) --kg per $output",
	D1	"Coal statistics (adjusted IEA) -- EJ",
	D2	"Crude Oil statistics (adjusted IEA)--EJ",
	D3	"Refined Oil statistics (adjusted IEA)--EJ",
	D4	"Natural gas statistics (adjusted IEA)--EJ",
	D5	"Electricity statistics (adjusted IEA)--TKWT",
	E1	"Basic Energy Prices",
	E2	"Implied Heat Rates (EJ Input to EJ/TKWH Output)",
	E3	"Cost Ratios (fuel input costs to output value)",
	E4	"Implicitly Determined Subsidy Rates",
	F1	"Crude Oil Prices ($ per GJ)",
	F2	"Refined Oil Prices ($ per GJ)",
	F3	"Natural Gas Prices ($ per GJ)",
	F4	"Coal Prices ($ per GJ)",
	F5	"Electricity Prices ($ per TKWH)",
	G1	"Energy value shares (%) -- rebalanced dataset",
	G2	"CO2 inventories (IEA)-- mton --  rebalanced dataset",
	G3	"Sectoral CO2 intensities --kg per $output Rebalanced dataset";

*	Read the GTAP dataset:

$batinclude ..\inclib\mrtdata iea

SET	IREP(I)	/ELY,I_S,CRP,NFM,NMM,T_T/;

*	----------------------------------------------------------
*	Set definitions for use with GTAP-E.SET:

SET	e(i) /	GAS	Natural gas works
		ELY	Electricity and heat 
		P_C	Refined oil products
		COL	Coal transformation
		OIL	Crude oil /,
	elec(i) electricity /ely/,
	nele(i) non-electric energy inputs /gas,p_c,col/
	oil(i)	refined oil /p_c/
	cru(i)  crude oil   /oil/,
	gas(i)	/gas/
	col(i)	/col/
	gov(i)	Sector representing government energy /ser/;

SET USR Energy users in the IEA dataset /
	I	Industry
	H	Households
	U	Utilities
	X	Export /;
 
*	----------------------------------------------------------

parameter	cecphys(*)	carbon emission coefficients for physical units 
				/col 0.025, gas 0.014, p_c 0.020 /;

*	Move public sector demand for energy into the production of 
*	public services, as it is treated in IEA:

ti(e,gov,r)$(vafm(e,gov,r)+vgm(e,r)) 
	= (vafm(e,gov,r)*ti(e,gov,r) + vgm(e,r)*tg(e,r)) / (vafm(e,gov,r)+vgm(e,r));
vafm(e,gov,r) = vafm(e,gov,r) + vgm(e,r);
vdgm(gov,r) = vdgm(gov,r) + sum(e, vgm(e,r)*(1 + tg(e,r)))/(1-ty(gov,r));
vigm(e,r) = 0;
vdgm(e,r) = 0;
vdfm(e,r) = vdm(e,r) - vdgm(e,r) - vdpm(e,r);
vifm(e,r) = vim(e,r) - vipm(e,r) - vigm(e,r);

tg(e,r) = 0;
vdm(gov,r) = ( sum(j, vafm(j,gov,r)*(1+ti(j,gov,r)))
           + sum(f,  vfm(f,gov,r)*(1+tf(f,gov,r)))) / (1-ty(gov,r)) - vxm(gov,r);


parameter VTWRMARGIN	Transport costs as fraction of delivered price;

vtwrmargin("total",e) = round(100 * sum((s,r), vtwr(e,s,r)) / sum(r, vim(e,r)));
vtwrmargin(r,e)$vim(e,r) = round(100 * sum(s, vtwr(e,s,r)) / vim(e,r));
vtwrmargin("total",e) = round(100 * sum((s,r), vtwr(e,s,r)) / sum(r, vim(e,r)));

option vtwrmargin:0;
display vtwrmargin;

*	Do an initial check of the GTAP equilibrium conditions:

$include chkeq

*	Read energy prices and quantities assembled by Gerard:

PARAMETER
	RGBP(R,E)	Basic energy price ($ per toe) -- 45x5
	TE(I,R,USR)	Energy tax rates by user"
	FVOL(R,E)	Final consumption volume (MTOE) -- 45x5
	IVOL(R,E,I)	Intermediate use volume -- 45x5x22
	MVOL(R,E)	Import volume (MTOE) -- 45x5 
	XVOL(R,E)	Export volume (MTOE) -- 45x5 ;

*	Read the energy price-quantity data:

$include ..\data\vail.dat

PARAMETERS
	EIND(I,I,R)	Industrial energy demand (EJ&TKWH),
	EFD(I,R)	Final energy demand (EJ&TKWH),
	EEXP(I,R)	Energy exports (EJ&TKWH),
	EIMP(I,R)	Energy imports (EJ&TKWH),
	PE(r,i,*)	Energy prices ($ per GJ or TKWH) 
	ETRADEBAL	Trade balance for energy volume data
	VSHARE		Energy value shares for non-energy sectors
	CRUDEINPUT	Crude oil inputs into non-refining uses
	ENERGY		Energy market statistics,
	EDOM(R,I)	Energy production for the domestic market
	ENERGYCOST	Aggregate energy cost data (exependiture gross of tax)
	CARBCOEF	Carbon coefficients for electric and non-electric energy
	CARBON		Direct carbon content for goods
	EPRICE		Basic energy prices (imputed - $ per GJ-TKWH)
	EXMARGIN	Energy Export Margin (% over export price) 
	TMBAR		Average tariff rate on energy imports, 
	TXBAR		Average tax rate on energy exports,
	VTWRCOEF	Trade transportation markup;


*	Extrade energy volume data in preferred units:

*		1 EJ = 1.0E9 GJ
*		1 EJ = 23.88 MTOE

*       Convert electricity flows to trillion kwh

*  1EJ  = 0.948E15 BTU
*  1kwh = 3413 BTU
*  1EJ  = 0.948e15/3413 kwh
*  1EJ  = 948/3413 trillion kwh

SCALAR  EJTKWH  EJ TO 10^12 KWH CONVERSION FACTOR;  EJTKWH=948/3413;

EIND(E,I,R) = round( IVOL(R,E,I)/23.88, 4);
EFD(E,R)    = round( FVOL(R,E)/23.88, 4);
EEXP(E,R)   = round( XVOL(R,E)/23.88, 4);
EIMP(E,R)   = round( MVOL(R,E)/23.88, 4);

EIND(elec,I,R) = round(EIND(elec,I,R)*EJTKWH,4); 
EFD(elec,R)    = round(EFD(elec,R)*EJTKWH,   4); 
EEXP(elec,R)   = round(EEXP(elec,R)*EJTKWH,  4); 
EIMP(elec,R)   = round(EIMP(elec,R)*EJTKWH,  4); 

B1(R,"IMPORTS") = ROUND(EIMP("COL",R),1);
B1(R,"EXPORTS") = ROUND(EEXP("COL",R),1);
B1(R,"DEMAND") = ROUND(EFD("COL",R) + SUM(I, EIND("COL",I,R)),1);
B1(R,"PRODUCTION") = ROUND( EFD("COL",R) + SUM(I, EIND("COL",I,R)) + EEXP("COL",R) - EIMP("COL",R),1);

LOOP(CRU,
  B2(R,"IMPORTS") = ROUND(EIMP(CRU,R),1);
  B2(R,"EXPORTS") = ROUND(EEXP(CRU,R),1);
  B2(R,"DEMAND") = ROUND(EFD(CRU,R) + SUM(I, EIND(CRU,I,R)),1);
  B2(R,"PRODUCTION") =ROUND(EFD(CRU,R) + SUM(I, EIND(CRU,I,R)) + EEXP(CRU,R) - EIMP(CRU,R),1);
);

LOOP(OIL,
  B3(R,"IMPORTS") = ROUND(EIMP(OIL,R),1);
  B3(R,"EXPORTS") = ROUND(EEXP(OIL,R),1);
  B3(R,"DEMAND") = ROUND(EFD(OIL,R) + SUM(I, EIND(OIL,I,R)),1);
  B3(R,"PRODUCTION") =ROUND( EFD(OIL,R) + SUM(I, EIND(OIL,I,R)) + EEXP(OIL,R) - EIMP(OIL,R),1);
);

LOOP(GAS,
  B4(R,"IMPORTS") = ROUND(EIMP(GAS,R),1);
  B4(R,"EXPORTS") = ROUND(EEXP(GAS,R),1);
  B4(R,"DEMAND") = ROUND(EFD(GAS,R) + SUM(I, EIND(GAS,I,R)),1);
  B4(R,"PRODUCTION") =ROUND(EFD(GAS,R) + SUM(I, EIND(GAS,I,R)) + EEXP(GAS,R) - EIMP(GAS,R),1);
);

LOOP(ELEC,
  B5(R,"IMPORTS") = ROUND(EIMP(ELEC,R),1);
  B5(R,"EXPORTS") = ROUND(EEXP(ELEC,R),1);
  B5(R,"DEMAND") = ROUND(EFD(ELEC,R) + SUM(I, EIND(ELEC,I,R)),1);
  B5(R,"PRODUCTION") =ROUND( EFD(ELEC,R) + SUM(I, EIND(ELEC,I,R)) + EEXP(ELEC,R) - EIMP(ELEC,R),1);
);

*	Having reported these flows we drop them from the dataset:

EEXP(elec,R)   = 0;
EIMP(elec,R)   = 0;

*	Check trade balance in the energy data:

ETRADEBAL(E) = SUM(R, EEXP(E,R)-EIMP(E,R) ) ;
DISPLAY ETRADEBAL;

*	Assess energy value shares:

vshare(r,i)$(vom(i,r) and (not e(i))) 
	= round( 100 * sum(e, vafm(e,i,r)*(1+ti(e,i,r))) / vom(i,r), 1);
option vshare:1;  display "Energy value shares implicit in original GTAP data:", vshare;

A1(R,IREP) = VSHARE(R,IREP);

CARBCOEF(R,NELE) = CECPHYS(NELE);
CARBCOEF(R,ELEC) = SUM(NELE, CECPHYS(NELE) * EIND(NELE,ELEC,R))/
			(sum(i, eind(elec,i,r)) + efd(elec,r));
C1(R,"TOTAL")=1000*SUM(E, CARBCOEF(R,E)*(SUM(I, EIND(E,I,R))+EFD(E,R)) );
C1(R,"INDUSTRIAL")=1000*SUM((E,I), CARBCOEF(R,E)*EIND(E,I,R));
C1(R,"FINAL")=1000*SUM(E, CARBCOEF(R,E)*EFD(E,R));
C1(R,"ELECTRICITY")=1000*SUM((ELEC,E), CARBCOEF(R,E)*EIND(E,ELEC,R));
C1(R,"CO2/GDP-kg/$")=C1(R,"TOTAL")/(10*SUM((F,I), VFM(F,I,R)));

C2(R,ELEC)$VOM(ELEC,R)=100*SUM(NELE, CECPHYS(NELE) * EIND(NELE,ELEC,R))/VOM(ELEC,R);
C2(R,IREP)$(NOT ELEC(IREP))=100*SUM(E, CARBCOEF(R,E)*EIND(E,IREP,R))/VOM(IREP,R);


*	Move all crude oil into refining:

loop(cru,

	crudeinput(r,i)$(not oil(i)) = eind(cru,i,r);
	crudeinput(r,"fd") = efd(cru,r);

	eind(cru,i,r)$(not oil(i)) = 0;
	efd(cru,r) = 0;

	eind(oil,i,r) = eind(oil,i,r) + crudeinput(r,i);
	efd(oil,r) = efd(oil,r) + crudeinput(r,"fd");
	eind(cru,oil,r) = eind(cru,oil,r) + sum(i, crudeinput(r,i)) + crudeinput(r,"fd");

);
display "OECD/IEA crude oil inputs:", crudeinput;

parameter	eprod	energy production levels
		tpes	total primary energy supply;


*-----------------------------------------------------------------------
*	Define NLP and MCP version of the EQBAL calibration model:
*-----------------------------------------------------------------------

variables       obj, edomz(e,r), eimpz(i,r), eindz(i,i,r), efdz(e,r), eexpz(e,r);

positive 
variables       edomz, eimpz, eindz, efdz, eexpz;

equations       ebalobj, esupply(e,r), euse(e,r), ebalance(e);

ebalobj..       obj =e=   sum((e,i,r), sqr(eindz(e,i,r)-eind(e,i,r)))
                        + sum((e,r),   sqr(efdz(e,r)   - efd(e,r)))
                        + sum((e,r),   sqr(eexpz(e,r)  - eexp(e,r)))
                        + sum((e,r),   sqr(eimpz(e,r)  - eimp(e,r)))
                        + sum((e,r),   sqr(edomz(e,r) + eexpz(e,r) - eprod(e,r)))

                        + sum((e,r)$(not eimp(e,r)),    1000 * eimpz(e,r))
                        + sum((e,r)$(not eexp(e,r)),    1000 * eexpz(e,r))
                        + sum((e,i,r)$(not eind(e,i,r)),1000 * eindz(e,i,r))
                        + sum((e,r)$(not efd(e,r)),     1000 * efdz(e,r));

esupply(e,r)..	edomz(e,r) + eimpz(e,r) =e= tpes(e,r);

euse(e,r)..	tpes(e,r) =e= sum(i, eindz(e,i,r)) + efdz(e,r);

ebalance(e)..	sum(r, eimpz(e,r)) =e= sum(r, eexpz(e,r));

model eqbal / ebalobj, esupply, euse, ebalance /;


alias(e,a_e);
alias(i,a_i);
alias(r,a_r);


equations
    d_edomz(a_e,a_r),
    d_eimpz(a_e,a_r),
    d_eindz(a_e,a_i,a_r),
    d_efdz(a_e,a_r),
    d_eexpz(a_e,a_r);

variables
    m_euse(e,r),
    m_esupply(e,r),
    m_ebalance(e);


d_edomz(a_e,a_r) .. 2*(edomz(a_e,a_r)+eexpz(a_e,a_r)-eprod(a_e,a_r)) 
			- m_esupply(a_e,a_r) =g= 0;

d_eimpz(a_e,a_r) .. 2*(eimpz(a_e,a_r)-eimp(a_e,a_r))+1000*(1$(not eimp(a_e,a_r)))

		 -m_esupply(a_e,a_r) -m_ebalance(a_e)  =g= 0;

d_eindz(a_e,a_i,a_r) .. 

	2*(eindz(a_e,a_i,a_r)-eind(a_e,a_i,a_r))+1000*(1$(not eind(a_e,a_i,a_r)))

	-m_euse(a_e,a_r)*(-1)  =g= 0;

d_efdz(a_e,a_r) .. 2*(efdz(a_e,a_r)-efd(a_e,a_r))+1000*(1$(not efd(a_e,a_r)))

	 -m_euse(a_e,a_r)*(-1)  =g= 0;

d_eexpz(a_e,a_r) .. 2*(eexpz(a_e,a_r)-eexp(a_e,a_r))+2*(edomz
	 (a_e,a_r)+eexpz(a_e,a_r)-eprod(a_e,a_r))+1000*(1$(not eexp(a_e,a_r)))
	 -m_ebalance(a_e)*(-1)  =g= 0;


model m_eqbal /
    d_edomz.edomz,
    d_eimpz.eimpz,
    d_eindz.eindz,
    d_efdz.efdz,
    d_eexpz.eexpz,
    euse.m_euse,
    esupply.m_esupply,
    ebalance.m_ebalance
/;

*---------------------- Extract prices and taxes --------------------------

*	Use the ECA data for basic prices:

PE(R,E,"basic")    = RGBP(r,e)*0.02388/10;
PE(R,ELEC,"basic") = PE(R,ELEC,"basic")/EJTKWH; 

*	Catch outliers:

PE(R,E,"BASIC") = MAX(0.1,PE(R,E,"BASIC"));

*	SCALE TO PROPER UNITS FOR DISPLAY:

PE(R,E,"BASIC") = PE(R,E,"BASIC") * 10;

OPTION PE:3:1:2;
DISPLAY PE;

*	Then scale back to GTAP basis, 10s of billions of dollars:

PE(R,E,"basic") = PE(R,E,"basic") / 10;

*	Quick fix:

set	nogas(r) / phl, ury, hkg, mar /;

eind("gas",j,nogas) = 0;
efd("gas",nogas) = 0;
eimp("gas",nogas) = 0;

eprod(elec,r) = round(eexp(elec,r) - eimp(elec,r) + sum(i, eind(elec,i,r)) + efd(elec,r), 3);
eprod(e,r)$(not elec(e)) = round(eexp(e,r) - eimp(e,r) + sum(i, eind(e,i,r)) + efd(e,r), 1);
display eprod;
eexp(e,r)$(not eprod(e,r)) = 0;

*	Record total primary energy here:

tpes(e,r) = sum(i, eind(e,i,r)) + efd(e,r);

*	Deal only with net production:

eprod(e,r) = max(0, eprod(e,r) - eind(e,e,r));

eind(e,e,r) = 0;
vafm(e,e,r) = 0;

eindz.fx(e,e,r) = 0;
edomz.fx(e,r)$(not eprod(e,r)) = 0;
eexpz.fx(e,r)$(not eprod(e,r)) = 0;
eindz.fx(i,e,r)$(not eprod(e,r)) = 0;

*.	Suppress electricity trade:

eexpz.fx("ely",r) = 0;
eimpz.fx("ely",r) = 0;


*	Omit crude oil imports to countries which do not refine
*	oil:

loop(oil, eimpz.fx(cru,r)$(not eprod(oil,r)) = 0; 
	  eimp(cru,r)$(not eprod(oil,r)) = 0; 
	  tpes(cru,r)$(not eprod(oil,r)) = 0;);

*	Also omit intermediate inputs of coal, gas and refined oil products
*	into non-electric activities:

eind(nele,e,r)$nele(e) = 0;
vafm(nele,e,r)$nele(e) = 0;
eindz.fx(nele,e,r)$nele(e) = 0;

*	Eliminate electricity inputs to all energy sectors:

eind(elec,e,r)		= 0;
vafm(elec,e,r)		= 0;
eindz.fx(elec,e,r)	= 0;

*	Eliminate the intermediate flows in refined oil, coal and natural gas:

eind(gas,cru,r) = 0;
eind(col,cru,r) = 0;
eind(oil,cru,r) = 0;

vafm(gas,cru,r) = 0;
vafm(col,cru,r) = 0;
vafm(oil,cru,r) = 0;

eindz.fx(i,j,r)$(not eind(i,j,r)) = 0;
efdz.fx(e,r)$(not efd(e,r)) = 0;


tpes(e,r)$(not (sum(i, eind(e,i,r)) + efd(e,r))) = 0;

*	Solve the matrix balancing problem here:

eqbal.holdfixed = 1;
option nlp=minos5;
if (not mcpformat, solve eqbal using nlp minimizing obj;);
if (mcpformat, solve m_eqbal using mcp;);

eindz.fx(i,j,r)$(not eind(i,j,r)) = 0;
efdz.fx(e,r)$(not efd(e,r)) = 0;




**** My commenting ************************
eqbal.workspace = 10;

if (mcpformat, solve m_eqbal using mcp;);
if (not mcpformat, solve eqbal using nlp minimizing obj;);




PARAMETER	GTAPCOEF	GTAP-based carbon coefficient;

GTAPCOEF(E,R)$(vdm(e,r)+vim(e,r)) = 1000 * TPES(E,R)*CECPHYS(E) / (vdm(e,r)+vim(e,r));
GTAPCOEF(ELEC,R) = SUM(E, GTAPCOEF(E,R) * VAFM(E,ELEC,R)) / VOM(ELEC,R);

A2(R,"TOTAL") = 1000 * SUM(E,   TPES(E,R)*CECPHYS(E))
			+SUM(ELEC, GTAPCOEF(ELEC,R)*(VDM(ELEC,R)+VIM(ELEC,R)));

A2(R,"INDUSTRIAL") = SUM(E, GTAPCOEF(E,R) * (vdfm(e,r)+vifm(e,r)));

A2(R,"FINAL")  = SUM(E, GTAPCOEF(E,R) * (vpm(e,r) + vgm(e,r)) );

A2(R,"ELECTRICITY") = SUM((ELEC,E), GTAPCOEF(E,R) * VAFM(E,ELEC,R) );

A2(R,"CO2/GDP-kg/$") = A2(R,"TOTAL") /(10*sum((f,i), vfm(f,i,r)));

A3(R,ELEC) = GTAPCOEF(ELEC,R)/10;
A3(R,IREP)$(NOT ELEC(IREP)) = SUM(E, GTAPCOEF(E,R) * VAFM(E,IREP,R)) /(10*VOM(IREP,R));

*	Extract the energy quantities:

eimp(e,r)   = eimpz.l(e,r);
eind(e,i,r) = eindz.l(e,i,r);
efd(e,r)    = efdz.l(e,r);
eexp(e,r)   = eexpz.l(e,r);

*	Generate a summary report:

energy(e,r,"imports")  = eimp(e,r);
energy(e,r,"exports")  = eexp(e,r);
energy(e,r,"demand")   = sum(i, eind(e,i,r)) + efd(e,r);
energy(e,r,"domestic") = round(sum(i,eind(e,i,r)) + efd(e,r) - eimp(e,r),6);
option energy:3:1:1;
display energy;

*	Compute domestic supplies:

edom(r,e) = round(sum(i, eind(e,i,r)) + efd(e,r) - eimp(e,r), 6);
display "Energy supply to the domestic market (original IEA dataset):", edom;

parameter	excessm	  Excess of imports over domestic demand plus export;
excessm(e,r) = max(0, eimp(e,r) - sum(i, eind(e,i,r)) - efd(e,r) - eexp(e,r));
display excessm;

etradebal(e) = sum(r, eexp(e,r)-eimp(e,r) ) ;
display etradebal;

abort$(smin((e,r),edom(r,e)) lt 0) "Implied edom is negative!",edom;
abort$(smin((e,r),eexp(e,r)) lt 0) "Implied eexp is negative!",eexp;
abort$(smin((e,r),eimp(e,r)) lt 0) "Implied eimp is negative!",eimp;

D1(R,"IMPORTS") = ROUND(EIMP("COL",R),1);
D1(R,"EXPORTS") = ROUND(EEXP("COL",R),1);
D1(R,"DEMAND") = ROUND(EFD("COL",R) + SUM(I, EIND("COL",I,R)),1);
D1(R,"PRODUCTION") =ROUND( EFD("COL",R) + SUM(I, EIND("COL",I,R)) + EEXP("COL",R) - EIMP("COL",R),1);

LOOP(CRU,
  D2(R,"IMPORTS") = ROUND(EIMP(CRU,R),1);
  D2(R,"EXPORTS") = ROUND(EEXP(CRU,R),1);
  D2(R,"DEMAND") = ROUND(EFD(CRU,R) + SUM(I, EIND(CRU,I,R)),1);
  D2(R,"PRODUCTION") =ROUND(EFD(CRU,R) + SUM(I, EIND(CRU,I,R)) + EEXP(CRU,R) - EIMP(CRU,R),1);
);

LOOP(OIL,
  D3(R,"IMPORTS") = ROUND(EIMP(OIL,R),1);
  D3(R,"EXPORTS") = ROUND(EEXP(OIL,R),1);
  D3(R,"DEMAND") = ROUND(EFD(OIL,R) + SUM(I, EIND(OIL,I,R)),1);
  D3(R,"PRODUCTION") =ROUND( EFD(OIL,R) + SUM(I, EIND(OIL,I,R)) + EEXP(OIL,R) - EIMP(OIL,R),1);
);

LOOP(GAS,
  D4(R,"IMPORTS") = ROUND(EIMP(GAS,R),1);
  D4(R,"EXPORTS") = ROUND(EEXP(GAS,R),1);
  D4(R,"DEMAND") = ROUND(EFD(GAS,R) + SUM(I, EIND(GAS,I,R)),1);
  D4(R,"PRODUCTION") =ROUND(EFD(GAS,R) + SUM(I, EIND(GAS,I,R)) + EEXP(GAS,R) - EIMP(GAS,R),1);
);

LOOP(ELEC,
  D5(R,"IMPORTS") = ROUND(EIMP(ELEC,R),1);
  D5(R,"EXPORTS") = ROUND(EEXP(ELEC,R),1);
  D5(R,"DEMAND") = ROUND(EFD(ELEC,R) + SUM(I, EIND(ELEC,I,R)),1);
  D5(R,"PRODUCTION") =ROUND( EFD(ELEC,R) + SUM(I, EIND(ELEC,I,R)) + EEXP(ELEC,R) - EIMP(ELEC,R),1);
);


parameter	eratio	some key ratios in energy production;

parameter nfoss_ele     Non-fossil share of electric supply /
AUS 0.038,    NZL 0.482,    JPN 0.375,    KOR 0.497,    IDN 0.167,    MYS 0.065
PHL 0.615,    THA 0.029,    VNM 0.296,    CHN 0.065,    TWN 0.438,    IND 0.091
LKA 0.535,    RAS 0.254,    CAN 0.630,    USA 0.275,    MEX 0.130,    CAM 0.250
VEN 0.342,    COL 0.329,    RAP 0.444,    ARG 0.346,    BRA 0.862,    CHL 0.550
URY 0.722,    RSM 0.582,    GBR 0.281,    DEU 0.324,    SWE 0.915,    FIN 0.623
REU 0.459,    EFT 0.967,    CEA 0.057,    FSU 0.548,    TUR 0.180,    MAR 0.038
RNF 0.074,    SAF 0.065,    RSA 0.065,    RSS 0.219,    ROW 0.281 /;


loop((cru,oil,elec),

  eratio(r,"price_cru") = 10 * pe(r,cru,"basic");
  eratio(r,"price_oil") = 10 * pe(r,oil,"basic");
  eratio(r,"price_elec") = 10 * pe(r,elec,"basic");
  eratio(r,"crude_oil")$(edom(r,oil)+eexp(oil,r)) = eind(cru,oil,r) / (edom(r,oil)+eexp(oil,r));
  eratio(r,"nele_ele")$(edom(r,elec)+eexp(elec,r)) = 
		sum(nele,eind(nele,elec,r)) / ((1-nfoss_ele(r)) * (edom(r,elec)+eexp(elec,r)));

  eratio(r,"cost_oil")$(edom(r,oil)+eexp(oil,r)) = 
	pe(r,cru,"basic")*eind(cru,oil,r) / (pe(r,oil,"basic")*(edom(r,oil)+eexp(oil,r)));
  eratio(r,"cost_ele")$(edom(r,elec)+eexp(elec,r)) = 
	sum(nele,pe(r,nele,"basic")*eind(nele,elec,r)) / (pe(r,elec,"basic")*(edom(r,elec)+eexp(elec,r)));
);
option eratio:1;
display eratio;

LOOP( (CRU,OIL,ELEC,GAS,COL),
  E1(R,"CRUDE") = 10 * pe(r,cru,"basic");
  E1(R,"REFINED") = 10 * pe(r,oil,"basic");
  E1(R,"ELECTRIC") = 10 * pe(r,ELEC,"basic");
  E1(R,"GAS") = 10 * pe(r,GAS,"basic");
  E1(R,"COAL") = 10 * pe(r,COL,"basic");
);
E2(R,"REFINING") = ERATIO(R,"CRUDE_OIL");
E2(R,"ELECTRICITY") = ERATIO(R,"NELE_ELE");

E3(R,"REFINING") = ERATIO(R,"COST_OIL");
E3(R,"ELECTRICITY") = ERATIO(R,"COST_ELE");

*------------ Compare aggregate value of energy demand ---------

ENERGYCOST(E,R,"GTAP")$(NOT CRU(E)) = SUM(J, VAFM(E,J,R)*(1+TI(E,J,R))) + 
						VPM(E,R)*(1+TP(E,R));

ENERGYCOST(E,R,"IEA/GERARD")$(NOT CRU(E)) = PE(R,E,"basic") * (
		SUM(J, EIND(E,J,R)*(1+TE(E,R,"I")$(NOT ELEC(J)) + TE(E,R,"U")$ELEC(J))) +
		EFD(E,R)*(1+TE(E,R,"H")) );

ENERGYCOST("Total",R,"GTAP")       = SUM(E, ENERGYCOST(E,R,"GTAP"));
ENERGYCOST("Total",R,"IEA/GERARD") = SUM(E, ENERGYCOST(E,R,"IEA/GERARD"));
ENERGYCOST("Total",R,"ADJUST%") = 100 * (ENERGYCOST("Total",R,"IEA/GERARD")/ENERGYCOST("Total",R,"GTAP") - 1);
OPTION ENERGYCOST:3:1:1;
DISPLAY ENERGYCOST;

vshare(r,i)$(vom(i,r) and (not e(i))) = round( 100 * 
  sum(e, pe(r,e,"basic")*eind(e,i,r)*(1 + te(e,r,"i")$(not elec(i)) 
					+ te(e,r,"u")$elec(i))) / vom(i,r), 1);
option vshare:1;
display "Energy value shares implicit in IEA price and quantity data:", vshare;

*	Evaluate implied carbon content associated with direct energy inputs:

carbcoef(r,nele) = cecphys(nele);
carbcoef(r,elec) = sum(nele, cecphys(nele) * eind(nele,elec,r));
carbon(r,i)$((vom(i,r) gt 0) and (not nele(i))) = round(1000 * sum(e, carbcoef(r,e)*eind(e,i,r)) / vom(i,r));
option carbon:0;
display carbon;

eprice(r,"nele_iea")$SUM(NELE, SUM(J,EIND(NELE,J,R)) + EFD(NELE,R))
		 = 10 * SUM(NELE, PE(R,NELE,"basic") * (
			SUM(J,	EIND(NELE,J,R)*(1+TE(NELE,R,"I")$(NOT ELEC(J)) + TE(NELE,R,"U")$ELEC(J))) 
			      + EFD(NELE,R)*(1+TE(NELE,R,"H")) ))
		     /  SUM(NELE, SUM(J,EIND(NELE,J,R)) + EFD(NELE,R));


eprice(r,"elec_iea")$SUM(ELEC, SUM(J,EIND(ELEC,J,R)) + EFD(ELEC,R))
		 = 10 * SUM(ELEC, PE(R,ELEC,"basic") * (
			SUM(J,	EIND(ELEC,J,R)*(1+TE(ELEC,R,"I")$(NOT ELEC(J)) + TE(ELEC,R,"U")$ELEC(J))) 
			      + EFD(ELEC,R)*(1+TE(ELEC,R,"H")) ))
		     /  SUM(ELEC, SUM(J,EIND(ELEC,J,R)) + EFD(ELEC,R));

eprice(r,"nele_gtap")$SUM(NELE, SUM(J,EIND(NELE,J,R)) + EFD(NELE,R))
		 = 10 * SUM(NELE,
			SUM(J,	VAFM(NELE,J,R)*(1+TI(NELE,J,R))) + 
				VPM(NELE,R)*(1+TP(NELE,R)) )
		     /  SUM(NELE, SUM(J,EIND(NELE,J,R)) + EFD(NELE,R));
			

eprice(r,"elec_gtap")$SUM(ELEC, SUM(J,EIND(ELEC,J,R)) + EFD(ELEC,R))
		 = 10 * SUM(ELEC,
			SUM(J,	VAFM(ELEC,J,R)*(1+TI(ELEC,J,R))) + 
				VPM(ELEC,R)*(1+TP(ELEC,R))  )
		     /  SUM(ELEC, SUM(J,EIND(ELEC,J,R)) + EFD(ELEC,R));


option eprice:1:1:1;
display eprice;

scalar qp  flag to solve the quadratic program;

set	noexport  Regions and goods for which IEA exports are positive and VXMD is zero, 
	noimport  Regions and goods for which IEA imports are positive and VXMD is zero;

noexport(r,e) = yes$(eexp(e,r) and (not vxm(e,r)));
noimport(r,e) = yes$(eimp(e,r) and (not vim(e,r)));
display noexport, noimport;

exmargin(r,e)$(eexp(e,r)*pe(r,e,"basic")) = round( 100 * (vxm(e,r)/(eexp(e,r)*pe(r,e,"basic")) - 1));
option exmargin:0;
display exmargin;

tmbar(e,r)$sum(s, vxmd(e,s,r)) = sum(s, vxmd(e,s,r)*tm(e,s,r)) / sum(s, vxmd(e,s,r));
txbar(e,r)$sum(s, vxmd(e,r,s)) = sum(s, vxmd(e,r,s)*tx(e,r,s)) / sum(s, vxmd(e,r,s));
tm(e,s,r)$(not vxmd(e,s,r)) = tmbar(e,r);
tx(e,r,s)$(not vxmd(e,r,s)) = txbar(e,r);

vtwrcoef(e,r,s)$vxmd(e,r,s) = vtwr(e,r,s)/vxmd(e,r,s);
vtwrcoef(e,r,s)$((sum(rr, vxmd(e,r,rr)+vxmd(e,rr,s)) and (not vxmd(e,r,s))))
	 = sum(rr, vtwr(e,r,rr)+vtwr(e,rr,s)) / sum(rr, vxmd(e,r,rr)+vxmd(e,rr,s));

option vtwrcoef:3:1:1;
display vtwrcoef, vtwr, vxmd;


*-----------------------------------------------------------------------
*	Define NLP and MCP version of the TBAL calibration model:
*-----------------------------------------------------------------------

variables       x(e,r,s)        Bilateral export flows
                pme(e,r)        Price level for imported energy
		pmeneg(e,r),
                obj             Objective function;

positive variable x, pme, vtwrp, vtwrn, pmeneg;

equations       export          export demand
                import          import supply
                vtwrsum         constraint on value of transport services
                relprice        constraint on relative prices of imports and exports
                equil           equilibrium condition
                objqp           defines the least-squares minimand;

export(e,r)$eexp(e,r)..         eexp(e,r) =g= sum(s$(eimp(e,s)), x(e,r,s));

import(e,r)$eimp(e,r)..         sum(s$eexp(e,s), x(e,s,r)) =g= eimp(e,r);

vtwrsum..                       sum((e,s,r), vtwrcoef(e,s,r) * pe(s,e,"basic") * x(e,s,r) )

                                        =e= sum((e,s,r), vtwr(e,s,r)) + vtwrp - vtwrn;

relprice(e,r)$eimp(e,r)..       pme(e,r) =g= pe(r,e,"basic") * (1 - pmeneg(e,r));

equil(e,r)$eimp(e,r)..          sum(s$eexp(e,s), ( (1 + tx(e,s,r)) + vtwrcoef(e,s,r)) 

	* pe(s,e,"basic") * x(e,s,r) * (1 + tm(e,s,r))) =e=  eimp(e,r) * pme(e,r);

objqp..              obj =e= 

*       Quadratic objective representing deviations from the GTAP
*       trade matrix:

        (sum((e,r,s)$(eexp(e,r) and eimp(e,s)), 

            sqr(pe(r,e,"basic")*x(e,r,s)-vxmd(e,r,s)) 
	      / (vxmd(e,r,s) + 0.01*eexp(e,r)$(not vxmd(e,r,s))))
	    )$(qp)

*       Linear penalty terms:

                + sum((e,r), 1000 * pmeneg(e,r))

                + sum((e,r,s)$(not vxmd(e,r,s)), 1000 * x(e,r,s))

                + 1000 * (vtwrp + vtwrn);

model tbal / export, import, relprice, vtwrsum, equil, objqp /;

alias(e,a_e);
alias(r,a_r);
alias(s,a_s);

equations
    d_x(a_e,a_r,a_s),
    d_pme(a_e,a_r),
    d_pmeneg(a_e,a_r),
    d_vtwrp,
    d_vtwrn;

variables
    m_equil(e,r),
    m_vtwrsum;

positive variables
    m_export(e,r),
    m_import(e,r),
    m_relprice(e,r);


d_x(a_e,a_r,a_s)$(x.lo(a_e,a_r,a_s) ne x.up(a_e,a_r,a_s)) .. 
  ((2*(pe(a_r,a_e,"basic")*x(a_e,a_r,a_s)
 -vxmd(a_e,a_r,a_s))*pe(a_r,a_e,"basic")*(1$(eexp(a_e,a_r) and
  eimp(a_e,a_s)))))$(qp)+1000*(1$(not vxmd(a_e,a_r,a_s)))

 -m_equil(a_e,a_s)*(1+tx(a_e,a_r,a_s)+vtwrcoef(a_e,a_r,a_s))*pe
 (a_r,a_e,"basic")*(1$(eexp(a_e,a_r)))*(1+tm(a_e,a_r,a_s))

 -m_vtwrsum*vtwrcoef(a_e,a_r,a_s)*pe(a_r,a_e,"basic")

 -m_export(a_e,a_r)*(-(1$(eimp(a_e,a_s))))

 -m_import(a_e,a_s)*1$(eexp(a_e,a_r)) 

 =g= 0;

d_pme(a_e,a_r)$(pme.lo(a_e,a_r) ne pme.up(a_e,a_r)) .. 

 -m_equil(a_e,a_r)*(-eimp(a_e,a_r))

 -m_relprice(a_e,a_r) 

 =g= 0;

d_pmeneg(a_e,a_r)$(pmeneg.lo(a_e,a_r) ne pmeneg.up(a_e,a_r)) ..

  1000 - m_relprice(a_e,a_r)*(-pe(a_r,a_e,"basic")*(-1)) 

 =g= 0;

d_vtwrp .. 1000

 -m_vtwrsum*(-1)*1 

 =g= 0;

d_vtwrn .. 1000

 -m_vtwrsum 

 =g= 0;


model m_tbal /
    d_x.x,
    d_pme.pme,
    d_pmeneg.pmeneg,
    d_vtwrp.vtwrp,
    d_vtwrn.vtwrn,
    equil.m_equil,
    vtwrsum.m_vtwrsum,
    export.m_export,
    import.m_import,
    relprice.m_relprice
/;

m_export.l(e,r) = 1;
m_import.l(e,r) = 1;

m_equil.fx(e,r)$(not (eimp(e,r))) = 0;

m_export.fx(e,r)$(not (eexp(e,r))) = 0;

m_import.fx(e,r)$(not (eimp(e,r))) = 0;

m_relprice.fx(e,r)$(not (eimp(e,r))) = 0;

*-----------------------------------------------------------------------

x.lo(e,r,s) = 0;
x.up(e,r,s) = +inf;
x.fx(e,r,r) = 0;
x.l(e,r,s)$pe(r,e,"basic") = vxmd(e,r,s) / pe(r,e,"basic");
x.fx(e,r,s)$((not eimp(e,s)) or (not eexp(e,r))) = 0;
x.fx("gas","fsu","usa") = 0;

*	Report starting point for the calibration:

parameter summary;
summary(r,"eimp") = eimp("ely",r);
summary(r,"eexp") = eexp("ely",r);
summary(r,"vxdm_imp") = sum(s, vxmd("ely",s,r));
summary(r,"vxdm_exp") = sum(s, vxmd("ely",r,s));
display summary;


pme.fx(e,r)$(not eimp(e,r)) = 1;
pmeneg.fx(e,r)$(not eimp(e,r)) = 0;

*	Rescale exports to maintain international balance:

eexp(e,r)$sum(rr,eexp(e,rr)) = eexp(e,r) * sum(rr, eimp(e,rr))/sum(rr,eexp(e,rr));

qp = 0;
option nlp=minos5;
solve tbal using nlp minimizing obj;

*	After determining a feasible basis, we fix the
*	non-zeros:

x.fx(e,r,s)$((not x.l(e,r,s)) and (not vxmd(e,r,s))) = 0;
vtwrp.fx = 0;
vtwrn.fx = 0;
pmeneg.up(e,r) = pmeneg.l(e,r);

qp = 1;
m_tbal.optfile=1;
if (mcpformat, solve m_tbal using mcp;);
if (not mcpformat, solve tbal using nlp minimizing obj;);

*	Compute import price indices:

pe(r,e,"import") = pme.l(e,r);

*	Recompute trade flows and trade costs:

vxmd(e,r,s) = x.l(e,r,s) * pe(r,e,"basic");
vtwr(e,r,s) = x.l(e,r,s) * pe(r,e,"basic") * vtwrcoef(e,r,s);

*	Generate some summary statistics:

parameter	vxmadj	Percentage change in the value of energy exports, 
		vimadj	Percentage change in the value of energy imports;

vxmadj(r,e)$vxm(e,r) = round( 100 * (sum(s, vxmd(e,r,s)) / vxm(e,r) - 1));
vimadj(r,e)$vim(e,r) = round( 100 * 
		(sum(s,(vxmd(e,s,r)*(1+tx(e,s,r))+vtwr(e,s,r))*(1+tm(e,s,r)))/vim(e,r) - 1));

option vxmadj:0, vimadj:0;
display vxmadj, vimadj;

parameter vtwrshare  International transport margins as a share of all trade costs (%);
vtwrshare(e) = 100 * sum((s,r), vtwr(e,s,r)) / sum((i,s,r), vtwr(i,s,r));
vtwrshare("total") = 100 * sum((e,s,r), vtwr(e,s,r)) / sum((i,s,r), vtwr(i,s,r))
display vtwrshare;

parameter energyxtax  Energy export tax rates (percentage);
energyxtax(e,r,s) = round(100 * tx(e,r,s));
option energyxtax:0;
display energyxtax;

*	Readjust supply and demand for transport services due to the potential
*	imbalanced introduced in the calibration process:

parameter yt0;
yt0("prod") = sum((i,r), vst(i,r));
yt0("demd") = sum((i,s,r),vtwr(i,s,r));
display yt0;
vst(i,r) = vst(i,r) *  yt0("demd") / yt0("prod") ;
vxm(i,r) = sum(s, vxmd(i,r,s)) + vst(i,r);

*	Construct energy prices and values:

parameter	ve	total value of energy supply, 
		eq	total quantity of electric and oil demand,
		thetam	value share for imports,
		edemand	energy demand net of electricity and oil refining inputs;

set ifd(*); ifd(i) = yes; ifd("fd") = yes; ifd(elec) = no; ifd(oil) = no;

edemand(r,e) = sum(i$((not elec(i)) and (not oil(i))), eind(e,i,r)) + efd(e,r);
ve(r,e) = pe(r,e,"basic")*edom(r,e) + pe(r,e,"import")*eimp(e,r);
eq(r,e) = sum(i$(elec(i) or oil(i)), eind(e,i,r));
thetam(e,r)$(edom(r,e)+eimp(e,r)) = eimp(e,r) / (edom(r,e)+eimp(e,r));

pe(r,e,i)$(oil(i) or elec(i)) = ( ve(r,e) / eq(r,e) )$((eq(r,e) gt 0) and (edemand(r,e) eq 0))
				+ min(pe(r,e,"basic"),pe(r,e,"import"))$(edemand(r,e) gt 0);
pe(r,e,ifd)$edemand(r,e) = ( ve(r,e) - sum(i$(oil(i) or elec(i)), pe(r,e,i)*eind(e,i,r)) ) / edemand(r,e);

LOOP(CRU,
F1(R,"BASIC") = 10 *  PE(R,CRU,"BASIC");
F1(R,"IMPORT") = 10 * PE(R,CRU,"IMPORT");
F1(R,"REFINERY") = 10 * SUM(OIL, PE(R,CRU,OIL));
);

LOOP(OIL,
F2(R,"BASIC") = 10 *  PE(R,OIL,"BASIC");
F2(R,"IMPORT") = 10 * PE(R,OIL,"IMPORT");
F2(R,"UTILITY") = 10 * SUM(ELEC, PE(R,OIL,ELEC));
F2(R,"MARKET") = 10 * PE(R,OIL,"FD");
);

LOOP(GAS,
F3(R,"BASIC") = 10 *  PE(R,GAS,"BASIC");
F3(R,"IMPORT") = 10 * PE(R,GAS,"IMPORT");
F3(R,"UTILITY") = 10 * SUM(ELEC, PE(R,GAS,ELEC));
F3(R,"MARKET") = 10 * PE(R,GAS,"FD");
);

LOOP(COL,
F4(R,"BASIC") = 10 *  PE(R,COL,"BASIC");
F4(R,"IMPORT") = 10 * PE(R,COL,"IMPORT");
F4(R,"UTILITY") = 10 * SUM(ELEC, PE(R,COL,ELEC));
F4(R,"MARKET") = 10 * PE(R,COL,"FD");
);

LOOP(ELEC,
F5(R,"BASIC") = 10 *  PE(R,ELEC,"BASIC");
F5(R,"IMPORT") = 10 * PE(R,ELEC,"IMPORT");
F5(R,"MARKET") = 10 * PE(R,ELEC,"FD");
);

vdm(e,r)  = edom(r,e) * pe(r,e,"basic");
vpm(e,r)  = efd(e,r)  * pe(r,e,"fd");

vipm(e,r) = min( vim(e,r), thetam(e,r)     * vpm(e,r));
vdpm(e,r) = vpm(e,r) - vipm(e,r);   

**** Old assignment ****
* vipm(e,r) = thetam(e,r)     * vpm(e,r);
* vdpm(e,r) = (1-thetam(e,r)) * vpm(e,r);
*******

vxm(e,r)  = sum(s, vxmd(e,r,s)) + vst(e,r);
vim(e,r)  = sum(s, (vxmd(e,s,r)*(1+tx(e,s,r)) + vtwr(e,s,r)) *(1+tm(e,s,r)) );
vom(i,r)  = vdm(i,r) + vxm(i,r);
vafm(e,i,r) = eind(e,i,r) * pe(r,e,i);
vdfm(e,r) = vdm(e,r) - vdpm(e,r);
vifm(e,r) = vim(e,r) - vipm(e,r);

*	Check transport cost shares again:

vtwrmargin("total",e)$sum(r, vim(e,r))
	 = round(100 * sum((s,r), vtwr(e,s,r)) / sum(r, vim(e,r)));
vtwrmargin(r,e) = 0;
vtwrmargin(r,e)$vim(e,r) = round(100 * sum(s, vtwr(e,s,r)) / vim(e,r));
option vtwrmargin:0;
display vtwrmargin;

vshare(r,i)$(vom(i,r) and (not e(i))) = round( 100 * sum(e, vafm(e,i,r)*(1+ti(e,i,r))) / vom(i,r), 1);
option vshare:1;
display vshare;

*	Assign tax rates on energy inputs:

ti(e,i,r)     = te(e,r,"i");
ti(e,elec,r)  = te(e,r,"u");
tp(e,r)       = te(e,r,"h");

*.	Change 3/99 -- apply the household tax on fuel
*	inputs to transportation.

ti(e,"t_t",r)     = te(e,r,"h");

*	Drop all non-energy inputs to energy production?

vafm(i,e,r)$(not e(i)) = 0;


*	Output energy balances:

parameter energy_bal energy balance;
energy_bal(r,e,"profit")$vom(e,r) = (1-ty(e,r)) - sum(i, vafm(i,e,r)*(1+ti(i,e,r)))/vom(e,r);
energy_bal(r,e,"output") = 0;
energy_bal(r,e,"ty_tax") = -ty(e,r);
energy_bal(r,e,i)$vom(e,r) = -vafm(i,e,r)*(1+ti(i,e,r))/vom(e,r);
display "Before calibration of subsidies:", energy_bal;

*	Subsidize electricity and oil refining activities where they are 
*	non-profitable:

parameter	vashare(i)	set lower bound on the value-added share in oil and electricity
		implsbd(r,i)	implicit energy production subsidy rate (% - calibrated to minimum value-added shares);

vashare(elec) = 0.4; 
vashare(oil)  = 0.2;

parameter	maxvid	maximum value of intermediate inputs
		vid	value of intermediate demand;

vid(r,e) = sum(i, vafm(i,e,r)*(1+ti(i,e,r)));
maxvid(r,e) = (1-vashare(e)) * vom(e,r);

display vid, maxvid;

*	Only apply the implicit subsidy to sectors in which the
*	implicit share of value-added is below vashare:

maxvid(r,e)$(vid(r,e) lt maxvid(r,e)) = 0;
display maxvid;


implsbd(r,e)$maxvid(r,e)
  = round(-100 * ( 1 - vid(r,e)/maxvid(r,e) )); 

LOOP((OIL,ELEC),
  E4(R,"REFINING")    = IMPLSBD(R,OIL);
  E4(R,"ELECTRICITY") = IMPLSBD(R,ELEC);
);

option implsbd:0;
display implsbd, ty, vom, vashare;

loop((e,r)$(vom(e,r) gt 0),
  if (vid(r,e)/((1-ty(e,r))*vom(e,r)) gt 1-vashare(e),
    ty(e,r) = 1 - vid(r,e) / ((1-vashare(e))*vom(e,r));
  );
);
energy_bal(r,e,"profit")$vom(e,r) = 1 - ty(e,r)  - vid(r,e)/vom(e,r);
energy_bal(r,e,"output") = 0;
energy_bal(r,e,"ty_tax") = -ty(e,r);
energy_bal(r,e,i)$vom(e,r) = -vafm(i,e,r)*(1+ti(i,e,r))/vom(e,r);
display "After calibration of subsidies (fractions):", energy_bal;

alias (f,ff);

*	Assign a resource input for those sectors where there is energy 
*	production according to IEA, yet there are no primary factor inputs
*	according to GTAP:

vfm(f,e,r)$(not vom(e,r)) = 0;
vfm("res",e,r)$(vom(e,r) and (not sum(ff, vfm(ff,e,r)*(1+tf(ff,e,r))))) = 1;
vfm(f,e,r)$sum(ff, vfm(ff,e,r)*(1+tf(ff,e,r)))
	= vfm(f,e,r) * (vom(e,r)*(1-ty(e,r)) - sum(i, vafm(i,e,r)*(1+ti(i,e,r))))
	/ sum(ff, vfm(ff,e,r)*(1+tf(ff,e,r)));

display "*** Check of imbalances in GTAP equilibrium condition prior to recalibration of IO tables:";
$include chkeq

parameter	marketchk(*,*,*)	cross-check on market subtotals;

*	Read the regional balancing program:

set rb(r)	dynamic set used to define the current region;
rb(r) = yes;

alias (r,reg);

*-----------------------------------------------------------------------
*	Define NLP and MCP version of the REGBAL calibration model:
*-----------------------------------------------------------------------
positive
variables       investz         calibrated invest
                vpmz(i)         calibrated vpm
                vgmz(i)         calibrated vgm
                vdmz(i)         calibrated vdm
                vafmz(j,i)      calibrated vadm
                vfmz(f,i)       calibrated vfm
                vmz(d,i)        calibrated vm
                vdz(d,i)        calibrated vd;

variable        obj;

equations       profit(i,r)     benchmark consistency - zero profit
                dommkt(i,r)     benchmark consistency - domestic market
                imports(i,r)    benchmark consistency - import supply
                interm(i,r)     benchmark consistency - intermediate demand
                consum(i,r)     benchmark consistency - consumer imports
                govern(i,r)     benchmark consistency - government imports

                objbal          defines the objective function;

objbal..        obj =e= 10 * sum(rb, sqr(investz-vdm("cgd",rb)) +
                        sum(i,   sqr(vpmz(i)-vpm(i,rb))
                        +        sqr(vgmz(i)-vgm(i,rb))
                        +        sqr(vdmz(i)-vdm(i,rb)) 
                        + sum(j, sqr(vafmz(j,i)-vafm(j,i,rb)))
                        + sum(f, sqr(vfmz(f,i)-vfm(f,i,rb)))
                        + sum(d, sqr(vmz(d,i)-vm(d,i,rb))
                        +        sqr(vdz(d,i)-vd(d,i,rb)))));

profit(i,rb)$(not e(i))..       (vdmz(i)+vxm(i,rb))*(1-ty(i,rb)) =e= sum(j,vafmz(j,i)*(1+ti(j,i,rb)))  
					+ sum(f, vfmz(f,i)*(1+tf(f,i,rb)));

dommkt(i,rb)$(not e(i))..       vdmz(i) =e= sum(d, vdz(d,i)) + investz$(cgd(i));

imports(i,rb)$(not e(i))..      sum(d, vmz(d,i)) =e= vim(i,rb);

interm(i,rb)$(not e(i))..       vdz("i",i) + vmz("i",i) =e= sum(j, vafmz(i,j));
consum(i,rb)$(not e(i))..       vdz("c",i) + vmz("c",i) =e= vpmz(i);
govern(i,rb)$(not e(i))..       vdz("g",i) + vmz("g",i) =e= vgmz(i);

model regbal /profit,dommkt,imports,interm,consum,govern,objbal/;

alias(d,a_d);
alias(i,a_i);
alias(f,a_f);
alias(j,a_j);


equations
    d_investz,
    d_vpmz(a_i),
    d_vgmz(a_i),
    d_vdmz(a_i),
    d_vafmz(a_j,a_i),
    d_vfmz(a_f,a_i),
    d_vmz(a_d,a_i),
    d_vdz(a_d,a_i);

variables
    m_govern(i,r),
    m_dommkt(i,r),
    m_profit(i,r),
    m_consum(i,r),
    m_imports(i,r),
    m_interm(i,r);


d_investz .. 10*SUM(rb,2*(investz-vdm("cgd",rb)))

 -SUM((i,rb),m_dommkt(i,rb)*(-(1$(cgd(i))))) 

 =g= 0;

d_vpmz(a_i) .. 10*SUM(rb,2*(vpmz(a_i)-vpm(a_i,rb)))

 -SUM(rb,m_consum(a_i,rb)*(-1)) 

 =g= 0;

d_vgmz(a_i) .. 10*SUM(rb,2*(vgmz(a_i)-vgm(a_i,rb)))

 -SUM(rb,m_govern(a_i,rb)*(-1)) 

 =g= 0;

d_vdmz(a_i) .. 10*SUM(rb,2*(vdmz(a_i)-vdm(a_i,rb)))

 -SUM(rb,m_dommkt(a_i,rb))

 -SUM(rb,m_profit(a_i,rb)*(1-ty(a_i,rb))) 

 =g= 0;

d_vafmz(a_j,a_i) .. 10*SUM(rb,2*(vafmz(a_j,a_i)-vafm(a_j,a_i
 ,rb)))

 -SUM(rb,m_profit(a_i,rb)*(-((1+ti(a_j,a_i,rb)))))

 -SUM(rb,m_interm(a_j,rb)*(-1)) 

 =g= 0;

d_vfmz(a_f,a_i) .. 10*SUM(rb,2*(vfmz(a_f,a_i)-vfm(a_f,a_i,rb
 )))

 -SUM(rb,m_profit(a_i,rb)*(-((1+tf(a_f,a_i,rb))))) 

 =g= 0;

d_vmz(a_d,a_i) .. 10*SUM(rb,2*(vmz(a_d,a_i)-vm(a_d,a_i,rb))
 )

 -SUM(rb$(sameas(a_d,"g")),m_govern(a_i,rb))

 -SUM(rb$(sameas(a_d,"c")),m_consum(a_i,rb))

 -SUM(rb,m_imports(a_i,rb))

 -SUM(rb$(sameas(a_d,"i")),m_interm(a_i,rb)) 

 =g= 0;

d_vdz(a_d,a_i) .. 10*SUM(rb,2*(vdz(a_d,a_i)-vd(a_d,a_i,rb))
 )

 -SUM(rb$(sameas(a_d,"g")),m_govern(a_i,rb))

 -SUM(rb,m_dommkt(a_i,rb)*(-1))

 -SUM(rb$(sameas(a_d,"c")),m_consum(a_i,rb))

 -SUM(rb$(sameas(a_d,"i")),m_interm(a_i,rb)) 

 =g= 0;


model m_regbal /
    d_investz.investz,
    d_vpmz.vpmz,
    d_vgmz.vgmz,
    d_vdmz.vdmz,
    d_vafmz.vafmz,
    d_vfmz.vfmz,
    d_vmz.vmz,
    d_vdz.vdz,
    govern.m_govern,
    dommkt.m_dommkt,
    profit.m_profit,
    consum.m_consum,
    imports.m_imports,
    interm.m_interm
/;

m_govern.fx(i,rb)$(not ((not e(i)))) = 0;
m_dommkt.fx(i,rb)$(not ((not e(i)))) = 0;
m_profit.fx(i,rb)$(not ((not e(i)))) = 0;
m_consum.fx(i,rb)$(not ((not e(i)))) = 0;
m_imports.fx(i,rb)$(not ((not e(i)))) = 0;
m_interm.fx(i,rb)$(not ((not e(i)))) = 0;

*-----------------------------------------------------------------------

rb(r) = no;
loop(reg,

*	Identify current region:

	rb(reg) = yes;

*	Set default bounds:

	vgmz.lo(i)   = 0;	vgmz.up(i)   = inf;
	vpmz.lo(i)   = 0;	vpmz.up(i)   = inf;
	vdmz.lo(i) = 0; 	vdmz.up(i) = inf;
	vafmz.lo(j,i) = 0;	vafmz.up(j,i) = inf;
	vfmz.lo(f,i) = 0;	vfmz.up(f,i) = inf;
	vmz.lo(d,i) = 0;	vmz.up(d,i) = inf;
	vdz.lo(d,i) = 0;	vdz.up(d,i) = inf;
	investz.lo = 0;		investz.up = inf;

*	Initial level values:

	vgmz.l(i) = vgm(i,reg);
	vpmz.l(i) = vpm(i,reg);
	vdmz.l(i) = vdm(i,reg);
	vafmz.l(j,i) = vafm(j,i,reg);
	vfmz.l(f,i) = vfm(f,i,reg);
	vmz.l(d,i) = vm(d,i,reg);
	vdz.l(d,i) = vd(d,i,reg);
	investz.l = vdm("cgd",reg);
	
*	Impose the same sparsity pattern as is present in the original dataset:

	vgmz.fx(i)$(not vgm(i,reg)) = 0;
	vpmz.fx(i)$(not vpm(i,reg)) = 0;
	vafmz.fx(j,i)$(not vafm(j,i,reg)) = 0;
	vfmz.fx( f,i)$(not vfm( f,i,reg)) = 0;
	vdmz.fx(i)$(not vdm(i,reg)) = 0;
	vmz.fx(d,i)$(not vm(d,i,reg)) = 0;
	vdz.fx(d,i)$(not vd(d,i,reg)) = 0;

*	Fix the energy demand:

	vafmz.fx(e,i) = vafm(e,i,reg);
	vpmz.fx(e)    = vpm(e,reg);
	vgmz.fx(e)    = 0;

	vdz.fx("g",e) = 0;
	vmz.fx("g",e) = 0;
	vdz.fx("c",e) = vdpm(e,reg);
	vmz.fx("c",e) = vipm(e,reg);
	vdz.fx("i",e) = vdfm(e,reg);
	vmz.fx("i",e) = vifm(e,reg);

*	Fix inputs to energy production:

	vfmz.fx(f,e)  = vfm(f,e,reg);
	vafmz.fx(i,e) = vafm(i,e,reg);
	vdmz.fx(e)    = vdm(e,reg);

	if (mcpformat, solve m_regbal using mcp;);
	if (not mcpformat, solve regbal using nlp minimizing obj;);

*	Retrieve the solution:

	vgm(i,reg)    = vgmz.l(i);
	vpm(i,reg)    = vpmz.l(i);
	vdm(i,reg)    = vdmz.l(i);
	vafm(j,i,reg) = vafmz.l(j,i);
	vfm(f,i,reg)  = vfmz.l(f,i);

	vm(d,i,reg)   = vmz.l(d,i);
	vd(d,i,reg)   = vdz.l(d,i);

	vipm(i,reg) = vm("c",i,reg);
	vigm(i,reg) = vm("g",i,reg);
	vdpm(i,reg) = vd("c",i,reg);
	vdgm(i,reg) = vd("g",i,reg);
	vifm(i,reg) = vm("i",i,reg);
	vdfm(i,reg) = vd("i",i,reg);

	marketchk(i,reg,"vdfm") = vdfm(i,reg);
	marketchk(i,reg,"vifm") = vifm(i,reg);
	marketchk(i,reg,"vafm") = sum(j, vafm(i,j,reg));
	marketchk(i,reg,"total") = round(vdfm(i,reg) + vifm(i,reg) - sum(j, vafm(i,j,reg)), 6);

	rb(reg) = no;

);

*	Assess energy value shares:

vshare(r,i)$(vom(i,r) and (not e(i))) 

	= round( 100 * sum(e, vafm(e,i,r)*(1+ti(e,i,r))) / vom(i,r), 1);

G1(R,IREP) = VSHARE(R,IREP);

CARBCOEF(R,NELE) = CECPHYS(NELE);
CARBCOEF(R,ELEC) = SUM(NELE, CECPHYS(NELE) * EIND(NELE,ELEC,R))/
			(sum(i, eind(elec,i,r)) + efd(elec,r));
G2(R,"TOTAL")=1000*SUM(E, CARBCOEF(R,E)*(SUM(I, EIND(E,I,R))+EFD(E,R)) );
G2(R,"INDUSTRIAL")=1000*SUM((E,I), CARBCOEF(R,E)*EIND(E,I,R));
G2(R,"FINAL")=1000*SUM(E, CARBCOEF(R,E)*EFD(E,R));
G2(R,"ELECTRICITY")=1000*SUM((ELEC,E), CARBCOEF(R,E)*EIND(E,ELEC,R));
G2(R,"CO2/GDP-kg/$")=G2(R,"TOTAL")/(10*SUM((F,I), VFM(F,I,R)));

G3(R,ELEC)$VOM(ELEC,R)=100*SUM(NELE, CECPHYS(NELE) * EIND(NELE,ELEC,R))/VOM(ELEC,R);
G3(R,IREP)$(NOT ELEC(IREP))=100*SUM(E, CARBCOEF(R,E)*EIND(E,IREP,R))/VOM(IREP,R);

display marketchk;
display "*** Check of imbalances in GTAP equilibrium condition after recalibration of IO tables:";
$include chkeq

*	-----------------------------------------------------------
*	Move the dataset into a zip archive:

$call 'copy ..\defines\iea.set ..\defines\gtap1000.set >nul'
$call 'zip -j ..\data\gtap1000 ..\defines\gtap1000.set >nul'
$call 'zip -j ..\data\gtap1000 ..\defines\iea.set >nul';
$call 'zip -j ..\data\gtap1000 ..\defines\iea.map >nul';
$batinclude ..\inclib\zip ..\data\ gtap1000 ty ti tf tx tm tg tp vafm vfm vxmd vtwr vst vdgm vigm vdpm vipm eind efd eexp eimp 

*	------------------------------------------------------------
*	Generate reports on energy statistics:

file kout /energy1000.rep/; kout.nd=1; kout.pw=124; put kout;

$batinclude ..\inclib\gams2tbl A1
$batinclude ..\inclib\gams2tbl A2
$batinclude ..\inclib\gams2tbl A3

$batinclude ..\inclib\gams2tbl B1
$batinclude ..\inclib\gams2tbl B2
$batinclude ..\inclib\gams2tbl B3
$batinclude ..\inclib\gams2tbl B4
$batinclude ..\inclib\gams2tbl B5

$batinclude ..\inclib\gams2tbl C1
$batinclude ..\inclib\gams2tbl C2

$batinclude ..\inclib\gams2tbl D1
$batinclude ..\inclib\gams2tbl D2
$batinclude ..\inclib\gams2tbl D3
$batinclude ..\inclib\gams2tbl D4
$batinclude ..\inclib\gams2tbl D5

$batinclude ..\inclib\gams2tbl E1
$batinclude ..\inclib\gams2tbl E2
$batinclude ..\inclib\gams2tbl E3
$batinclude ..\inclib\gams2tbl E4

$batinclude ..\inclib\gams2tbl F1
$batinclude ..\inclib\gams2tbl F2
$batinclude ..\inclib\gams2tbl F3
$batinclude ..\inclib\gams2tbl F4
$batinclude ..\inclib\gams2tbl F5

$batinclude ..\inclib\gams2tbl G1
$batinclude ..\inclib\gams2tbl G2
$batinclude ..\inclib\gams2tbl G3

