$title  impose.gms: GAMS code which imposes tax rates

$include impose.inc

*       Begin by reading the GTAP set definitions from the zip archive:

$batinclude ..\inclib\unzip %source% set

$if not exist ..\defines\%target%.set unzip -oqCj ..\data\%source% %source%.set -d ..\defines\%target%.set >nul'
$call 'zip -j ..\data\%target% ..\defines\%target%.set';
$call 'zip -j ..\data\%target% ..\defines\%source%.set';
$if exist ..\defines\%source%.map $call 'zip -j ..\data\%target% ..\defines\%source%.map';

alias (i,j,k), (r,s,ss);

SET     cgd(i) /cgd/;

SET     D Classes of demand for which import shares are differentiated / 
        G       Public, 
        C       Private Consumption,
        I       Intermediate input /;

*       -----------------------------------------------------------
*       Read data from the archive:

parameter
        ty(i,r)         Output tax
        ti(j,i,r)       Intermediate input tax
        tf(f,i,r)       Factor tax
        tx(i,s,r)       Export tax rate (defined on a net basis)
        tm(i,s,r)       Import tariff rate
        tg(i,r)         Tax rates on government demand
        tp(i,r)         Tax rate on private demand

        vafm(j,i,r)     Aggregate intermediate inputs
        vfm(f,i,r)      Value of factor inputs (net of tax)
        vxmd(i,r,s)     Value of commodity trade (fob - net export tax)
        vtwr(i,r,s)     Transport services
        vst(i,r)        Value of international transport sales
        vdgm(i,r)       Government demand (domestic)
        vigm(i,r)       Government demand (imported)
        vdpm(i,r)       Private demands (domestic)
        vipm(i,r)       Private demands (imported);

$batinclude ..\inclib\unzip %source% gms

*       -----------------------------------------------------------
*       Compute intermediate values:

parameter
        vim(i,r)        Total value of imports (gross tariff)
        vxm(i,r)        Value of export (net excise tax)
        vdm(i,r)        Value of domestic output (net excise tax)
        vdfm(i,r)       Aggregate intermediate demand (domestic)
        vifm(i,r)       Aggregate intermediate demand (imported)
        market(i,r)     Market clearance condition
        vpm(i,r)        Aggregate private demand
        vgm(i,r)        Aggregate public demand;


*       Infer exports and imports from the trade matrix, 
*       domestic production from exhaustion of product:

vxm(i,r) = sum(s, vxmd(i,r,s)) + vst(i,r);
vim(i,r) = sum(s, (vxmd(i,s,r) * (1+tx(i,s,r)) + vtwr(i,s,r) ) * (1+tm(i,s,r)) );
vdm(i,r) = (sum(j, vafm(j,i,r)*(1+ti(j,i,r))) + sum(f, vfm(f,i,r)*(1+tf(f,i,r))))/(1-ty(i,r)) - vxm(i,r);
vdfm(i,r) = vdm(i,r) - vdgm(i,r) - vdpm(i,r) - vdm(i,r)$cgd(i);
vifm(i,r) = vim(i,r) - vipm(i,r) - vigm(i,r);

*       -----------------------------------------------------------
parameter 
        vimn    VXMD row sums before filtering, 
        yt0     Transport demand subtotals;

parameter vp, vg;

vpm(i,r) = vipm(i,r) + vdpm(i,r);
vpm(i,r) = vipm(i,r) + vdpm(i,r);
vp(r) = sum(i, vpm(i,r)*(1+tp(i,r)));

vdpm(i,r)$(not vpm(i,r)) = 0;
vipm(i,r)$(not vpm(i,r)) = 0;
tp(i,r)$(not vpm(i,r)) = 0;


vgm(i,r) = vigm(i,r) + vdgm(i,r);
vgm(i,r) = vigm(i,r) + vdgm(i,r);
vg(r) = sum(i, vgm(i,r)*(1+tg(i,r)));
vdgm(i,r)$(not vgm(i,r)) = 0;
vigm(i,r)$(not vgm(i,r)) = 0;
tg(i,r)$(not vgm(i,r)) = 0;

*       -----------------------------------------------------------
*       Check the original equilibrium:

$include chkeq

*       -----------------------------------------------------------
*       Read the revisions:

$include ..\defines\%target%.def

*       -----------------------------------------------------------
*       Recalibrate the dataset:

$include recalib

*       -----------------------------------------------------------
*       Check the revised equilibrium:

$include chkeq

*       Store everything in the new zip archive:

$if defined eind $goto energy

$batinclude ..\inclib\zip ..\data\ %target% ty ti tf tx tm tg tp vafm vfm vxmd vtwr vst vdgm vigm  vdpm vipm 
execute 'zip -j ..\data\%target% ..\defines\%target%.set';

$goto end

$label energy

* Sergey's mess (added annexb to the next line to suit GTAP-EG)

$batinclude ..\inclib\zip ..\data\ %target% ty ti tf tx tm tg tp vafm vfm vxmd vtwr vst vdgm vigm  vdpm vipm eind efd eexp eimp annexb

$label end

*       Generate a GDP report for the resulting dataset:

parameter
        pmx0(i,s,r)     Reference price of imports,
        pmt0(i,s,r)     Reference price of transport services,
        pg0(i,r)        Reference price of government demand,
        pc0(i,r)        Reference price for private demand,
        py0(i,r)        Reference output price (domestic or export),
        pai0(j,i,r)     Reference price for intermediate inputs,
        pf0(f,i,r)      Reference price for factor inputs;

pg0(i,r) = 1 + tg(i,r);
pc0(i,r) = 1 + tp(i,r);
pmt0(i,s,r) = (1 + tm(i,s,r));
pmx0(i,s,r) = (1 + tm(i,s,r)) * (1 + tx(i,s,r));
py0(i,r) =1 - ty(i,r);
pf0(f,j,s) = 1 + tf(f,j,s);

$include ..\inclib\gdpreport
